
#include <QtGui>
#include "igu.h"
#include "uploadthread.h"

IGU::IGU(QWidget *parent)
{
    setupUi( this ); // sets up GUI

    thread = new uploadThread;
    connect(this, SIGNAL( setLoginData(const QString&, const QString, const QString&) ), thread, SLOT( setLoginData(const QString&, const QString&, const QString&) ) );
    connect(this, SIGNAL( setCategory(const int&) ), thread, SLOT(setCategory(const int&) ) );
    connect(this, SIGNAL( setResize(const int&) ), thread, SLOT( setResize(const int&) ) );
    connect(this, SIGNAL( setResizeData(const int&, const int&) ), thread, SLOT( setResizeData(const int&, const int&) ) );
    connect(this, SIGNAL( setFilelistData( QStandardItemModel &) ), thread, SLOT( setFilelistData(QStandardItemModel &) ) );
    //connect(this, SIGNAL( setImageData( QImage &) ), thread, SLOT( setImageData(QImage &) ) );
    connect(this, SIGNAL( startUpload() ), thread, SLOT( startUpload() ) );

    connect(this, SIGNAL(getCategories()), thread, SLOT(getCategories()));
    connect(thread, SIGNAL(updateCategories(const QByteArray &)), this, SLOT(updateCategories(const QByteArray &)));
    //connect(this, SIGNAL(setData(const QString&, const QString, const QString&, const int&, const int&, const int&, const int&, QStandardItemModel &)), thread, SLOT(setData(const QString&, const QString&, const QString&, const int&, const int&, const int&, const int&, QStandardItemModel &)));

// filelist
    this->filelistmodel = new QStandardItemModel(0, 4, parent);
    filelistmodel->setHeaderData(0, Qt::Horizontal, QObject::tr("file"));
    filelistmodel->setHeaderData(1, Qt::Horizontal, QObject::tr("Comment"));
    filelistmodel->setHeaderData(2, Qt::Horizontal, QObject::tr("full path"));
    filelistmodel->setHeaderData(3, Qt::Horizontal, QObject::tr("Image"));

    filelist->setModel(filelistmodel);
    filelist->hideColumn(3);
    //filelist->setIconSize(QSize(50,50));
//    filelist->header()->setResizeMode(QHeaderView::Interactive);
//    filelist->header()->setResizeMode(1,QHeaderView::Stretch);


    this->catlistmodel =  new QStandardItemModel(0, 2, parent);
    catlist->setModel(catlistmodel);
    catlist->header()->hide();
    catlist->hideColumn(1);
// signals
    connect(website, SIGNAL( textChanged( const QString &) ), this, SLOT( websiteChanged(const QString &) ) );
    connect(username, SIGNAL( textChanged( const QString &) ), this, SLOT( usernameChanged(const QString &) ) );
    connect(password, SIGNAL( textChanged( const QString &) ), this, SLOT( passwordChanged(const QString &) ) );
    connect(maxWidth, SIGNAL( textChanged( const QString &) ), this, SLOT( maxWidthChanged(const QString &) ) );
    connect(maxHeight, SIGNAL( textChanged( const QString &) ), this, SLOT( maxHeightChanged(const QString &) ) );

    connect(category, SIGNAL( valueChanged( const int &) ), this, SLOT( categoryChanged(const int &) ) );
    connect(checkresize, SIGNAL( stateChanged( const int &) ), this, SLOT( checkresizeChanged(const int &) ) );

    connect(butadd, SIGNAL(pressed()), this, SLOT(addFiles()));
    connect(butaddclipboard, SIGNAL(pressed()), this, SLOT(addFilesFromClipboard()));
    connect(butdel, SIGNAL(pressed()), this, SLOT(removeFiles()));
    connect(filelist->selectionModel(), SIGNAL(selectionChanged ( const QItemSelection &, const QItemSelection & ) ), this, SLOT( filelistSelectionChanged ( const QItemSelection &, const QItemSelection & ) ));
   connect(catlist, SIGNAL(clicked(QModelIndex)), this, SLOT( catlistClicked(QModelIndex) ));

    connect(butupload, SIGNAL(pressed()), this, SLOT(uploadFilelist()));
    connect(butcat, SIGNAL(pressed()), this, SLOT(butcatPressed()));
// Settings
    //QSettings *settings = new QSettings(QSettings::IniFormat, QSettings::UserScope, "listui", "listui");
    this->settings = new QSettings( "igu.ini", QSettings::IniFormat );
    //settings->setValue("website", "http://azud.az.funpic.de");
    category->setValue( settings->value( "category" , "" ).toInt() );
    website->setText( settings->value( "website" , "" ).toString() );
    username->setText( settings->value( "username" , "" ).toString() );
    password->setText( settings->value( "password" , "" ).toString() );
    maxWidth->setText( settings->value( "maxwidth" , "1000" ).toString() );
    maxHeight->setText( settings->value( "maxheight" , "1000" ).toString() );
    if( settings->value( "resize" , 0 ).toInt() ){
        checkresize->setCheckState( Qt::Checked );
    } else {
        checkresize->setCheckState( Qt::Unchecked );
    }

    addFilesfromArguments();
    butcatPressed();
}

void IGU::addFilesfromArguments()
{
    QStringList args = QCoreApplication::arguments ();
    if(args.count() < 2 ){
        return;
    }
    QList<QByteArray> formats = QImageReader::supportedImageFormats();
    QString file, ext;
    QString niceName;
    int pos, rc;
    for (int i = 1; i < args.count(); ++i) {
        QStandardItemModel *mod = filelistmodel;
        file = args.at(i);

        ext = file.toLower();
        pos = ext.lastIndexOf(".");
        if (pos != -1)
            ext.remove(0, pos + 1);
        if(formats.indexOf(ext.toAscii()) == -1){
            continue;
        }
        rc = mod->invisibleRootItem ()->rowCount();
        mod->insertRow(rc);
        niceName = file;
        niceName.replace("\\", "/");
        pos = niceName.lastIndexOf("/");
        if (pos != -1)
            niceName.remove(0, pos + 1);

        mod->setData( mod->index(rc, 0) , niceName );
        mod->setData( mod->index(rc, 1) , "" );
        mod->setData( mod->index(rc, 2) , file );
        mod->itemFromIndex(mod->index(rc, 0))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
//        QIcon icon;
//        icon.addFile(file, QSize(50,50));
//        mod->itemFromIndex(mod->index(rc, 0))->setIcon(icon);
        mod->itemFromIndex(mod->index(rc, 2))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);

    }
}

void IGU::uploadFilelist()
{
    emit setLoginData(settings->value( "website" , "" ).toString(),
                     settings->value( "username" , "" ).toString(),
                     settings->value( "password" , "" ).toString()
                     );

    emit setCategory( settings->value( "category" , "" ).toInt() );

    emit setResize(settings->value( "resize" , "" ).toInt() );

    emit setResizeData(settings->value( "maxwidth" , "" ).toInt(),
                       settings->value( "maxheight" , "" ).toInt()
                      );

    emit setFilelistData( *filelistmodel );

    emit startUpload();

//emit setData(settings->value( "website" , "" ).toString(),
//                 settings->value( "username" , "" ).toString(),
//                 settings->value( "password" , "" ).toString(),
//                 settings->value( "category" , "" ).toInt(),
//                 settings->value( "resize" , "" ).toInt(),
//                 settings->value( "maxwidth" , "" ).toInt(),
//                 settings->value( "maxheight" , "" ).toInt(),
//                 *filelistmodel);
}

void IGU::addFiles()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::ExistingFiles);
    if ( settings->value("directory", "").toString() != "" ){
        dialog.setDirectory(settings->value("directory", "").toString());
    }
    dialog.setFilter( tr("Images (*.jpg *.gif *.bmp *.png *.tif *.tiff *.xpm *.svg)") );
    QStringList files;
    if (dialog.exec()){
        files = dialog.selectedFiles();
        settings->setValue("directory", dialog.directory().path());
        QStandardItemModel *mod = (QStandardItemModel*)filelist->model();
        foreach (QString file, files) {
            int rc = mod->invisibleRootItem ()->rowCount();
            mod->insertRow(rc);
            QString niceName = file;
            niceName.replace("\\", "/");
            int pos = niceName.lastIndexOf("/");
            if (pos != -1)
                niceName.remove(0, pos + 1);

            mod->setData( mod->index(rc, 0) , niceName );
            mod->setData( mod->index(rc, 1) , "" );
            mod->setData( mod->index(rc, 2) , file );
            mod->itemFromIndex(mod->index(rc, 0))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
            mod->itemFromIndex(mod->index(rc, 2))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
        }
    }
}

void IGU::addFilesFromClipboard()
{
    QClipboard *clipboard = QApplication::clipboard();
    if( clipboard->image().isNull() ){
        QMessageBox::information(0,"IGU", tr("no image found"));
        return;
    }
    QImage *img = new QImage();
    *img = clipboard->image();
    QStandardItemModel *mod = (QStandardItemModel*)filelist->model();
    int rc = mod->invisibleRootItem ()->rowCount();
    mod->insertRow(rc);

    mod->setData( mod->index(rc, 0) , "clipboard.jpg" );
    mod->setData( mod->index(rc, 1) , "" );
    mod->setData( mod->index(rc, 3) , *img );
    delete img;
    //mod->itemFromIndex(mod->index(rc, 0))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
    mod->itemFromIndex(mod->index(rc, 2))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
    mod->itemFromIndex(mod->index(rc, 3))->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
}

void IGU::removeFiles()
{
    QStandardItemModel *mod = (QStandardItemModel*)filelist->model();
    QModelIndexList rows = filelist->selectionModel()->selectedRows();
    //QMessageBox::information(0, tr("Systray"), QString::number(rows.count() ) );
    for (int i = rows.size()-1; i >= 0 ; i--) {
        mod->removeRow(rows.at(i).row(), rows[i].parent());
    }
}
/////////////////////////////
void IGU::butcatPressed()
{
    emit setLoginData(settings->value( "website" , "" ).toString(),
                     settings->value( "username" , "" ).toString(),
                     settings->value( "password" , "" ).toString()
                     );
    emit getCategories();
}

void IGU::updateCategories(const QByteArray &ba)
{
    qDebug()<<"updateCategories";
    //<td>- <a href="?gallery-S1">Member-Fotos</a></td>
    QRegExp rx("<td>([;nbsp& -]*)<a href=\".gallery-S([0-9]+)\">(.*)</a></td>");
    rx.setMinimal(true);
    QString content(ba);
    //qDebug()<<content;
//
    QList<QStandardItem*> parents;
    QList<int> indentations;
    parents << catlistmodel->invisibleRootItem();
    indentations << 0;

    int pos = 0;
    catlistmodel->setRowCount(0);

//
    while ((pos = rx.indexIn(content, pos)) != -1) {
        int position = rx.cap(1).count();

        if (!rx.cap(3).isEmpty()) {

            if (position > indentations.last()) {
                if (parents.last()->rowCount() > 0) {
                    parents << parents.last()->child(parents.last()->rowCount()-1);
                    indentations << position;
                }
            } else {
                while (position < indentations.last() && parents.count() > 0) {
                    parents.pop_back();
                    indentations.pop_back();
                }
            }
            QStandardItem *parent = parents.last();
            QList<QStandardItem*> items;
            QStandardItem *item = new QStandardItem(rx.cap(3));
            QStandardItem *item2 = new QStandardItem(rx.cap(2));
            items << item << item2;
            parent->appendRow(items);
            parent->child(parent->rowCount() - 1)->setData(rx.cap(2));
        }

         pos += rx.matchedLength();
    }
}
///////////////////////

void IGU::filelistSelectionChanged ( const QItemSelection &  selected, const QItemSelection & deselected )
{
    QStandardItemModel *mod = (QStandardItemModel*)filelist->model();
    QModelIndexList rows = filelist->selectionModel()->selectedRows();
    if(selected.count() & rows.count() == 1){
        if( mod->index(rows.at(0).row(), 2).data().toString().isEmpty() ){
            //qDebug() << "blblblblbl" <<mod->index(rows.at(0).row(), 3).data();
            imgview->setPixmap( QPixmap( ).fromImage(qvariant_cast<QImage> (mod->index(rows.at(0).row(), 3).data())).scaled(100,100, Qt::KeepAspectRatio) );
        }else{
            imgview->setPixmap( QPixmap( mod->index(rows.at(0).row(), 2).data().toString() ).scaled(100,100, Qt::KeepAspectRatio) );
        }
    }
}

void IGU::catlistClicked(QModelIndex ind)
{
    category->setValue(ind.parent().child(ind.row(),1).data().toInt());
    //QModelIndexList rows = catlist->selectionModel()->selectedRows();
//    if(selected.count() ){
//        qDebug() <<rows.at(0).row() <<catlistmodel->index(rows.at(0).row(), 1).data().toInt();
//        category->setValue(catlistmodel->index(rows.at(0).row(), 1).data().toInt());
//        category->setValue(catlistmodel->data(index(selected.row(),1)).toInt());
//    }
}

void IGU::websiteChanged( const QString & str )
{
    settings->setValue( "website", str );
    glink->setText(QString( "<a href=\"%1/index.php?gallery-%2\">%3</a>" ).arg( settings->value( "website" , "" ).toString(), settings->value( "category" , "" ).toString(), tr("Gallery") ) );
    gadminlink->setText(QString( "<a href=\"%1/admin.php?gallery-S%2\">%3</a>" ).arg( settings->value( "website" , "" ).toString(), settings->value( "category" , "" ).toString(), tr("Admin Gallery") ) );


}

void IGU::usernameChanged( const QString & str )
{
    settings->setValue( "username", str );
}

void IGU::passwordChanged( const QString & str )
{
    settings->setValue( "password", str );
}

void IGU::maxWidthChanged( const QString & val )
{
    settings->setValue( "maxwidth", val );
}

void IGU::maxHeightChanged( const QString & val )
{
    settings->setValue( "maxheight", val );
}

void IGU::categoryChanged( const int & cat )
{
    settings->setValue("category", cat );
    glink->setText(QString( "<a href=\"%1/index.php?gallery-%2\">%3</a>" ).arg( settings->value( "website" , "" ).toString(), settings->value( "category" , "" ).toString(), tr("Gallery") ) );
    gadminlink->setText(QString( "<a href=\"%1/admin.php?gallery-S%2\">%3</a>" ).arg( settings->value( "website" , "" ).toString(), settings->value( "category" , "" ).toString(), tr("Admin Gallery") ) );

}

void IGU::checkresizeChanged( const int & state)
{
    settings->setValue( "resize", state );
}
